<?php

namespace FiloBlu\OrderFlowRectifier\Model\Rectifiers;

use FiloBlu\OrderFlowRectifier\Api\RectifierInterface;

class OrderPaymentDataIncompleteRectifier implements RectifierInterface
{
    const ADYEN_NOTIFICATION_EVENT_CODE = 'AUTHORISATION';
    const PAYMENT_INFO_TO_CHECK = array(
        'pspReference' => 'pspReference',
        'adyen_avs_result' => 'avsResult',
        'adyen_cvc_result' => 'cvcResult',
        'adyen_total_fraud_score' => 'totalFraudScore',
        'adyen_refusal_reason_raw' => 'refusalReasonRaw',
        'adyen_acquirer_reference' => 'acquirerReference',
        'adyen_auth_code' => 'authCode',
        'adyen_card_bin' => 'cardBin',
        'adyen_expiry_date' => 'expiryDate'
    );

    protected $connection;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $connection
    )
    {
        $this->connection = $connection->getConnection();
    }

    public function getInfo()
    {
        return 'Adyen transaction data retrieval for blocked orders in order_exported';
    }

    public function getTitle()
    {
        return 'Info update related to the adyen transaction if missing';
    }

    public function executeSingle($order)
    {
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');
        $payment_additional_information = unserialize($order->additional_information);
        $adyen_additional_data = unserialize($order->additional_data);
        $orderPspReference = $order->pspreference;
        $dataToUpdate = array();

        if (is_null($order->cc_last_4) && !is_null($adyen_additional_data['cardSummary']))
        {
            $dataToUpdate['cc_last_4'] = $adyen_additional_data['cardSummary'];
        }

        if (is_null($order->cc_trans_id))
        {
            $dataToUpdate['cc_trans_id'] = $orderPspReference;
        }

        if (is_null($order->adyen_psp_reference))
        {
            $dataToUpdate['adyen_psp_reference'] = $orderPspReference;
        }

        $payAdditionalInfoUpdated = false;
        foreach (self::PAYMENT_INFO_TO_CHECK as $payInfo => $adyenData)
        {
            if(!array_key_exists($payInfo, $payment_additional_information))
            {
                if($payInfo == 'pspReference')
                {
                    $payment_additional_information[$payInfo] = $orderPspReference;
                    $payAdditionalInfoUpdated = true;
                }
                else
                {
                    $payment_additional_information[$payInfo] =
                        isset($adyen_additional_data[$adyenData]) ? $adyen_additional_data[$adyenData] : NULL;
                    $payAdditionalInfoUpdated = true;
                }
            }
        }

        if($payAdditionalInfoUpdated)
        {
            $dataToUpdate['additional_information'] = serialize($payment_additional_information);
        }

        $affectedRows = 0;

        if(!empty($dataToUpdate))
        {
            $affectedRows = $this->connection->update(
            $orderPaymentTable,
            $dataToUpdate,
            ['entity_id = ?' => $order->payment_id, 'parent_id = ?' => $order->parent_id]);
        }

        return $affectedRows > 0;
    }

    public function loadOrders()
    {
        $orderTable = $this->connection->getTableName('sales_order');
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');
        $adyenNotification = $this->connection->getTableName('adyen_notification');
        $now = new \DateTime();
        $limitDate = $now->sub(new \DateInterval('PT2H'));

        $query = "SELECT o.increment_id, o.entity_id, op.entity_id as payment_id, op.parent_id,
                         op.cc_last_4, op.cc_trans_id, op.additional_information, op.adyen_psp_reference,
                         an.entity_id as notification_id, an.pspreference, an.additional_data
FROM {$orderTable} o
INNER JOIN {$orderPaymentTable} op ON o.entity_id = op.parent_id AND op.method IN ('adyen_cc', 'adyen_oneclick')
INNER JOIN {$adyenNotification} an ON o.increment_id = an.merchant_reference
WHERE 
      o.`status` = 'order_exported' AND
      o.created_at <= '".$limitDate->format('Y-m-d H:i:s')."' AND (
      op.adyen_psp_reference IS NULL OR
      op.cc_last_4 IS NULL OR
      op.cc_trans_id IS NULL OR
      op.additional_information NOT LIKE '%pspReference%') AND
      an.event_code = '".self::ADYEN_NOTIFICATION_EVENT_CODE."' AND
      an.success = 'true'
ORDER BY o.entity_id DESC";

        $ordersToBeRectified = $this->connection->fetchAll($query, [], \Zend_Db::FETCH_OBJ);
        return $ordersToBeRectified;
    }
}